@@ -1,3 +1,4 @@
+import asyncio
 import json
 import os
 import traceback
@@ -9,11 +10,16 @@ SESSION_TOKEN = os.environ.get("AIOGPT_SESSIONTOKEN")
 
 
 class Chat:
+    conversation_id = None
+    parent_message_id = None
+    access_token = None
+    current_message = None
+    history = []
+    semaphore = asyncio.Semaphore(1)
+    API_URL = "https://chat.openai.com/backend-api/conversation"
+
     def __init__(self, session_token):
         self.session_token = session_token
-        self.access_token = None
-        self.history = []
-        self.API_URL = "https://chat.openai.com/backend-api/conversation"
         self.reset()
 
     @property
@@ -35,11 +41,14 @@ class Chat:
         self.parent_message_id = str(uuid.uuid4())
 
     async def say(self, message):
-        response = await self._say(message)
-        if not response:
-            await self.refresh()
+        async with self.semaphore:
+            self.current_message = "..."
             response = await self._say(message)
-        return response
+            if not response:
+                await self.refresh()
+                response = await self._say(message)
+            self.current_message = None
+            return response
 
     async def _say(self, message):
         if not self.access_token:
@@ -63,23 +72,22 @@ class Chat:
                 self.API_URL,
                 data=json.dumps(data),
             ) as resp:
-                response = await resp.text()
-        try:
-            raw = json.loads((response.splitlines()[-4]).split(":", 1)[1])
-            output = "\n".join(raw["message"]["content"]["parts"])
-            self.conversation_id = raw["conversation_id"]
-            self.parent_message_id = raw["message"]["id"]
-            self.history.append([self.conversation_id, message, output])
-            return (
-                output,
-                self.conversation_id,
-                self.parent_message_id,
-            )
-        except:
-
-            traceback.print_exc()
-            print(response)
-            return None
+                # loop through resp.content.readline()
+                while line := await resp.content.readline():
+                    line = line.decode("utf-8")
+                    if "data" not in line or "[DONE]" in line:
+                        continue
+                    try:
+                        raw = json.loads(line.split(":", 1)[1].strip())
+                        self.current_message = "\n".join(
+                            raw["message"]["content"]["parts"]
+                        )
+                        self.conversation_id = raw["conversation_id"]
+                        self.parent_message_id = raw["message"]["id"]
+                    except:
+                        traceback.print_exc()
+                        print(line)
+            return (self.current_message, self.conversation_id, self.parent_message_id)
 
     async def refresh(self):
         cookies = {
@@ -102,9 +110,11 @@ class Chat:
                     return None
 
 
-if __name__ == "__main__":
+async def main():
     chat = Chat(SESSION_TOKEN)
-    chat.reset()
-    import asyncio
+    print(chat.say("Hello."))
 
-    asyncio.run(chat.say("hello"))
+
+if __name__ == "__main__":
+    loop = asyncio.get_event_loop()
+    asyncio.run(main())
